/*
  Copyright (c) 2023 PIXILAB Technologies AB, Sweden (http://pixilab.se). All Rights Reserved.
*/

#include <Encoder.h>

// The encoder will count from 0 to MAX_POS.
const int MAX_POS = 2000;

// Create an encoder connected to pin 2 and 3.
Encoder encoder(2, 3);

long prevPos = 0;
unsigned long prevTime = 0;

const int bufferSize = 32;
char readBuffer[bufferSize];
int readIndex = 0;

void setup() {
  // Set the baud rate to 57600 on startup.
  Serial.begin(57600);
}

void loop() {
  // Read the current encoder position.
  long pos = encoder.read();
  // Read the current time.
  unsigned long time = millis();

  // If the encoder position is greater than MAX_POS, reset the encoder
  // position to MAX_POS.
  if (pos > MAX_POS) {
    pos = MAX_POS;
    encoder.write(pos);
  }
  // If the encoder position is less than 0, reset the encoder position to 0.
  else if (pos < 0) {
    pos = 0;
    encoder.write(pos);
  }

  // If (at least) 20 ms have passed, send the encoder position.
  if (time - prevTime >= 20) {
    if (pos != prevPos) {
      double normalized = (double)pos / (double)MAX_POS;
      int decimals = 5;

      Serial.println(normalized, decimals);
      prevPos = pos;
      prevTime = time;
    }
  }
  
  // If serial data is available, reset the position to the received value.
  while (Serial.available() > 0) {
    char c = Serial.read();

    if (c == '\n') {
      double normalized = atof(readBuffer);
      long newPos = normalized * MAX_POS;
      
      encoder.write(newPos);
      readIndex = 0;
    }
    else {
      if (readIndex >= bufferSize) {
        readIndex = bufferSize - 1;
      }
      readBuffer[readIndex++] = c;
    }
  }
}
